@@ -32,7 +32,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 '''
 from __future__ import division
 import numpy as np
-from osgeo import gdal, gdalnumeric, ogr, osr
+from osgeo import gdal, gdalnumeric, ogr, osr, gdal_array
 from gdalconst import *
 from skimage.measure import block_reduce
 import matplotlib.pyplot as plt
@@ -69,7 +69,7 @@ def map_pixel(point_x, point_y, cellx, celly, xmin, ymax):
 	point_y=np.array(point_y)
 	col = np.around((point_x - xmin) / cellx).astype(int)
 	row = np.around((point_y - ymax) / celly).astype(int)
-	return col,row
+	return row,col
 
 # Aggregate raster to higher resolution using sums
 def aggregate(raster,NDV,block_size):
@@ -138,16 +138,16 @@ def align_rasters(raster,alignraster,how=np.mean,cxsize=None,cysize=None,masked=
 		araster=np.ma.masked_array(araster, mask=araster==NDV2, fill_value=NDV2)
 		amin=araster.min()
 		if GeoT1[0]<=GeoT2[0]:
-			mcol,row3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+			row3,mcol=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
 			acol=0
 		else:
-			acol,row3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+			row3,acol=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
 			mcol=0
 		if GeoT1[3]<=GeoT2[3]:
-			col3,arow=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+			arow,col3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
 			mrow=0
 		else:
-			col3,mrow=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+			mrow,col3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
 			arow=0
 		'''
 		col3,row3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
@@ -219,7 +219,10 @@ class GeoRaster():
             geot: GDAL Geotransformation
             nodata_value: No data value in raster, optional
         '''
-        self.raster = raster
+        if isinstance(raster, np.ma.core.MaskedArray):
+            self.raster = raster
+        else:
+            self.raster = np.ma.masked_array(raster, mask= raster==nodata_value, fill_value=nodata_value)
         self.geot = geot
         self.nodata_value = nodata_value
         self.shape = raster.shape
@@ -338,11 +341,14 @@ class GeoRaster():
          geo.datatype = 'gdal.GDT_'+type
         '''
         if self.datatype is None:
-            self.datatype = type(self.raster.data[0,1])
-            if self.datatype.find('int')!=-1:
-                self.datatype = gdal.GDT_Int32
-            else:
-                self.datatype = gdal.GDT_Float64
+            self.datatype = gdal_array.NumericTypeCodeToGDALTypeCode(self.raster.data.dtype)
+            if self.datatype is None:
+                if self.raster.data.dtype.name.find('int')!=-1:
+                    self.raster = self.raster.astype(np.int32)
+                    self.datatype = gdal_array.NumericTypeCodeToGDALTypeCode(self.raster.data.dtype)
+                else:
+                    self.raster = self.raster.astype(np.float64)
+                    self.datatype = gdal_array.NumericTypeCodeToGDALTypeCode(self.raster.data.dtype)
         create_geotiff(filename, self.raster, gdal.GetDriverByName('GTiff'), self.nodata_value, self.shape[1], self.shape[0], self.geot, self.projection, self.datatype)
 
     def plot(self):
@@ -517,13 +523,26 @@ class GeoRaster():
         '''
         return self.raster.var(*args, **kwargs)
 
+    def count(self, *args, **kwargs):
+        '''
+        geo.count(axis=None)
+        Count the non-masked elements of the array along the given axis.
+        '''
+        return self.raster.count(*args, **kwargs)
+
     def gini(self):
-        """Return computed Gini coefficient.
         """
-        xsort = sorted(self.raster.data[self.raster.data!=self.nodata_value].flatten()) # increasing order
-        y = np.cumsum(xsort)
-        B = sum(y) / (y[-1] * len(xsort))
-        return 1 + 1./len(xsort) - 2*B
+        geo.gini()
+        
+        Return computed Gini coefficient.
+        """
+        if self.count()>1:
+            xsort = sorted(self.raster.data[self.raster.data!=self.nodata_value].flatten()) # increasing order
+            y = np.cumsum(xsort)
+            B = sum(y) / (y[-1] * len(xsort))
+            return 1 + 1./len(xsort) - 2*B
+        else:
+            return 1
 
     def flatten(self, *args, **kwargs):
         '''
@@ -558,7 +577,7 @@ class GeoRaster():
         
         Return value of raster in location 
         '''
-        col, row =map_pixel(point_x, point_y, self.x_cell_size, self.y_cell_size, self.xmin, self.ymax)
+        row, col =map_pixel(point_x, point_y, self.x_cell_size, self.y_cell_size, self.xmin, self.ymax)
         return self.raster[row, col]
 
     def extract(self, point_x, point_y, radius=0):
@@ -568,11 +587,11 @@ class GeoRaster():
         Return subraster of raster geo around location (x,y) with radius r
         where (x,y) and r are in the same coordinate system as geo
         '''
-        col, row =map_pixel(point_x, point_y, self.x_cell_size, self.y_cell_size, self.xmin, self.ymax)
-        col2 = self.x_cell_size/radius
-        row2 = self.y_cell_size/radius
-        return GeoRaster(self.raster[max(row-row2, 0):min(row+row2, self.shape[0]), \
-                        max(col-col2, 0):min(col+col2, self.shape[1])], self.geot, nodata_value = self.nodata_value)
+        row, col =map_pixel(point_x, point_y, self.x_cell_size, self.y_cell_size, self.xmin, self.ymax)
+        col2 = np.abs(radius/self.x_cell_size)
+        row2 = np.abs(radius/self.y_cell_size)
+        return GeoRaster(self.raster[max(row-row2, 0):min(row+row2+1, self.shape[0]), \
+                        max(col-col2, 0):min(col+col2+1, self.shape[1])], self.geot, nodata_value = self.nodata_value)
 
 # Union of rasters
 def union(rasters):
@@ -592,10 +611,10 @@ def union(rasters):
         lonmax = max([i.xmax for i in rasters])
         latmin = min([i.ymin for i in rasters])
         latmax = max([i.ymax for i in rasters])
-        shape = (np.round((latmax-latmin)/rasters[0].x_cell_size).astype(int),np.round((lonmax-lonmin)/rasters[0].x_cell_size).astype(int))
+        shape = (np.abs(np.round((latmax-latmin)/rasters[0].y_cell_size)).astype(int),np.round((lonmax-lonmin)/rasters[0].x_cell_size).astype(int))
         out = ndv*np.ones(shape)
         for i in rasters:
-            (col,row) = map_pixel(i.xmin, i.ymax, rasters[0].x_cell_size, rasters[0].y_cell_size, lonmin, latmax)
+            (row,col) = map_pixel(i.xmin, i.ymax, rasters[0].x_cell_size, rasters[0].y_cell_size, lonmin, latmax)
             out[row:row+i.shape[0],col:col+i.shape[1]] = np.where(i.raster.data!=i.nodata_value, i.raster.data,\
                                                          out[row:row+i.shape[0],col:col+i.shape[1]])#i.raster
         return GeoRaster(out, (lonmin, rasters[0].x_cell_size, 0.0, latmax, 0.0, rasters[0].y_cell_size), nodata_value=ndv)
@@ -621,3 +640,17 @@ def from_file(filename):
     data = gdalnumeric.LoadFile(filename)
     data = np.ma.masked_array(data, mask=data==NDV,fill_value=NDV)
     return GeoRaster(data,GeoT, nodata_value=NDV, projection=Projection, datatype=DataType)
+
+# GDAL conversion types
+NP2GDAL_CONVERSION = {
+  "uint8": 1,
+  "int8": 1,
+  "uint16": 2,
+  "int16": 3,
+  "uint32": 4,
+  "int32": 5,
+  "float32": 6,
+  "float64": 7,
+  "complex64": 10,
+  "complex128": 11,
+}
